import mysql.connector
from mysql.connector import Error

class Barang:
    def __init__(self, conn, id_barang: int, nama_barang: str, stok: int, deskripsi: str, foto: str, status_pengiriman: str) -> None:
        self.connection = conn
        self.__id_barang = id_barang
        self.__nama_barang = nama_barang
        self.__stok = stok
        self.__deskripsi = deskripsi
        self.__foto = foto
        self.__status_pengiriman = status_pengiriman

    def get_id_barang(self) -> int:
        return self.__id_barang
    
    def get_nama_barang(self) -> str:
        return self.__nama_barang
    
    def get_stok(self) -> int:
        return self.__stok
    
    def get_deskripsi(self) -> str:
        return self.__deskripsi
    
    def get_foto(self) -> str:
        return self.__foto
    
    def get_status_pengiriman(self) -> str:
        return self.__status_pengiriman
    
    def set_id_barang(self, id_barang: int) -> None:
        self.__id_barang = id_barang

    def set_nama_barang(self, nama_barang: str) -> None:
        self.__nama_barang = nama_barang

    def set_stok(self, stok: int) -> None:
        self.__stok = stok

    def set_deskripsi(self, deskripsi: str) -> None:
        self.__deskripsi = deskripsi

    def set_foto(self, foto: str) -> None:
        self.__foto = foto

    def set_status_pengiriman(self, status_pengiriman: str) -> None:
        self.__status_pengiriman = status_pengiriman

    @classmethod
    def max_capacity(self, conn, id_warehouse) -> int:
        cursor = conn.cursor()
        cursor.execute("SELECT kapasitas_maksimum FROM warehouse WHERE id_warehouse = %s", (id_warehouse,))
        max_cap = cursor.fetchone()
        return max_cap
    ''
    @classmethod
    def now_capacity(self, conn, id_warehouse) -> int:
        cursor = conn.cursor()
        cursor.execute("SELECT SUM(stok) FROM barang WHERE id_warehouse = %s", (id_warehouse,))
        cur_cap = cursor.fetchone()
        return cur_cap
    
    @classmethod
    def now_in_warehouse_capacity(self, conn, id_warehouse) -> int:
        cursor = conn.cursor()
        cursor.execute("SELECT kapasitas_terkini FROM warehouse WHERE id_warehouse = %s", (id_warehouse,))
        cur_cap = cursor.fetchone()
        return cur_cap if cur_cap and cur_cap[0] is not None else (0,)
    
    @classmethod
    def now_stok_barang(self, conn, id_warehouse, id_barang) -> int:
        cursor = conn.cursor()
        cursor.execute("SELECT stok FROM barang WHERE id_warehouse = %s AND id_barang = %s", (id_warehouse,id_barang,))
        stok_barang = cursor.fetchone()
        return stok_barang
    
    @classmethod
    def stok_all(self, conn, id_warehouse) -> int:
        cursor = conn.cursor()
        cursor.execute("SELECT SUM(stok) FROM barang WHERE id_warehouse = %s", (id_warehouse,))
        stok_barang = cursor.fetchone()
        return stok_barang
    
    @classmethod
    def loadable_stok(self, conn, dest_warehouse, stok_num) -> bool:
        cursor = conn.cursor()
        cursor.execute("SELECT kapasitas_maksimum FROM warehouse WHERE id_warehouse = %s", (dest_warehouse,))
        kapasitas_maksimum = cursor.fetchone()[0]

        cursor.execute("SELECT SUM(stok) FROM barang WHERE id_warehouse = %s", (dest_warehouse,))
        dest_capacity = cursor.fetchone()[0] or 0

        if dest_capacity + stok_num > kapasitas_maksimum:
            return False
        return True

    
    @classmethod
    def id_exist(self, conn, id_barang) -> bool:
        cursor = conn.cursor()
        query = "SELECT COUNT(*) FROM barang WHERE id_barang = %s"
        cursor.execute(query, (id_barang,))
        if cursor.fetchone()[0] > 0:
            return True
        return False

    @classmethod
    # cek stok dan kapasitas dilakukan di luar fungsi
    def add(self, conn, id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto) -> None:
        cursor = conn.cursor()
        if foto is None:
            query = "INSERT INTO barang (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, status_pengiriman) VALUES (%s, %s, %s, %s, %s, %s)"
            values = (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, "Berada di Warehouse")
        else:
            query = "INSERT INTO barang (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto, status_pengiriman) VALUES (%s, %s, %s, %s, %s, %s, %s)"
            values = (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto, "Berada di Warehouse")
        cursor.execute(query, values)
        # print("Barang berhasil ditambahkan.")
        conn.commit()
        cursor.close()
        # conn.close()

    @classmethod
    def delete(self, conn, id_warehouse, id_barang):
        cursor = conn.cursor()
        query = "DELETE FROM Barang WHERE id_barang = %s AND id_warehouse = %s"
        values = (id_barang, id_warehouse)
        cursor.execute(query, values)
        conn.commit()
        cursor.close()
        # conn.close()

    @classmethod
    def update(self, connection, id_warehouse, id_barang, updates):
        cursor = connection.cursor()
        query_parts = [f"{key} = %s" for key in updates.keys()]
        query = f"UPDATE barang SET {', '.join(query_parts)} WHERE id_warehouse = %s AND id_barang = %s"
        values = tuple(updates.values()) + (id_warehouse, id_barang,)
        cursor.execute(query, values)
        cursor.close()

    @classmethod
    def update_curr_cap(self, connection, id_warehouse, cap_update):
        cursor = connection.cursor()
        query = f"UPDATE warehouse SET kapasitas_terkini = %s WHERE id_warehouse = %s"
        values = (cap_update, id_warehouse,)
        cursor.execute(query, values)
        cursor.close()

    @classmethod
    def search(self, conn, id_warehouse, part_name):
        cursor = conn.cursor(dictionary=True)
        query = "SELECT * FROM barang WHERE id_warehouse = %s AND nama_barang LIKE %s"
        values = (id_warehouse, '%' + part_name + '%',)
        cursor.execute(query, values)
        results = cursor.fetchall()
        cursor.close()
        return results
    
    @classmethod
    def find_by_id(self, conn, id_warehouse, id_barang):
        cursor = conn.cursor()
        query = "SELECT * FROM barang WHERE id_warehouse = %s AND id_barang = %s"
        cursor.execute(query, (id_warehouse, id_barang,))
        result = cursor.fetchone()
        
        if result:
            columns = [desc[0] for desc in cursor.description]
            barang_data = dict(zip(columns, result))
            return barang_data
        return None
    
    @classmethod
    def find_status(self, conn, id_warehouse, id_barang):
        cursor = conn.cursor()
        query = "SELECT status_pengiriman FROM barang WHERE id_warehouse = %s AND id_barang = %s"
        cursor.execute(query, (id_warehouse, id_barang,))
        result = cursor.fetchone()
        return result[0] if result else None
    
    @classmethod
    def pindah(self, conn, curr_warehouse, id_barang, dest_warehouse):
        cursor = conn.cursor()
        if self.find_status(conn, curr_warehouse, id_barang) == 'Dalam Pengiriman':
            return False
        else:
            query = "UPDATE barang SET id_warehouse = %s, status_pengiriman = 'Dalam Pengiriman' WHERE id_barang = %s AND id_warehouse = %s AND status_pengiriman = 'Berada di Warehouse'"
            values = (dest_warehouse, id_barang, curr_warehouse)
            cursor.execute(query, values)
            conn.commit()
            return True

    @classmethod
    def pindah_semua(self, conn, curr_warehouse, dest_warehouse):
        cursor = conn.cursor()
        query = "UPDATE barang SET id_warehouse = %s, status_pengiriman = 'Dalam Pengiriman' WHERE id_warehouse = %s AND status_pengiriman = 'Berada di Warehouse'"
        values = (dest_warehouse, curr_warehouse)
        cursor.execute(query, values)
        conn.commit()