import mysql.connector
from mysql.connector import Error
from decimal import Decimal
from classes.Barang import Barang
from classes.ListBarang import ListBarang

class BarangController:
    def __init__(self, host='localhost', database='warehosea', user='root', password='170611'):
        self.host = host
        self.database = database
        self.user = user
        self.password = password

    def connect(self):
        """Establish a database connection and return the connection object."""
        try:
            conn = mysql.connector.connect(
                host=self.host,
                database=self.database,
                user=self.user,
                password=self.password,
                auth_plugin='mysql_native_password'
            )
            if conn.is_connected():
                return conn
        except Error as e:
            print(f"Error connecting to MySQL: {e}")

    def get_barang_by_id(self, barang_id):
        conn = self.connect()
        if conn:
            try:
                cursor = conn.cursor(dictionary=True)
                query = "SELECT * FROM barang WHERE id_barang = %s"
                cursor.execute(query, (barang_id,))
                result = cursor.fetchone()
                return result
            finally:
                cursor.close()
                conn.close()
        else:
            print("Failed to establish database connection.")
            return None

    def lihat_daftar_barang(self, id_warehouse):
        conn = self.connect()
        if conn:
            results = ListBarang.get_list_barang(conn, id_warehouse)
            conn.close()
            return results

    def tambah_barang(self, id_warehouse, ID, nama, stok, deskripsi, foto_path):
        conn = self.connect()
        if conn:
            try:
                add_stock = int(stok)
                if (not Barang.id_exist(conn, ID)):
                    if Barang.loadable_stok(conn, id_warehouse, add_stock):
                        Barang.add(conn, id_warehouse, ID, nama, deskripsi, stok, foto_path)
                        return True, "Barang berhasil ditambahkan."
                    else:
                        return False, "Kapasitas warehouse tidak mencukupi untuk menampung barang."
                else: 
                    return False, "ID sudah ada sebelumnya."
            finally:
                conn.close()
        return False, "Failed to connect to the database."

    def hapus_barang(self, id_warehouse, id_barang):
        conn = self.connect()
        if conn:
            try:
                Barang.delete(conn, id_warehouse, id_barang)
                conn.commit()
            except Error as e:
                print(f"Error deleting item: {e}")
                conn.rollback()
            finally:
                conn.close()
    
    def update_barang(self, id_warehouse, id_barang, updates):
        conn = self.connect()
        if conn:
            try:
                barang_data = Barang.find_by_id(conn, id_warehouse, id_barang)
                if not barang_data:
                    return False, "Barang tidak ditemukan"

                current_stok = int(barang_data['stok'])
                new_stok = int(updates.get('stok', current_stok))

                if new_stok != current_stok:
                    additional_stok = new_stok - current_stok
                    # print(f"Change in stock: {additional_stok}")
                    if not Barang.loadable_stok(conn, id_warehouse, additional_stok):
                        return False, 'Kapasitas warehouse tidak mencukupi untuk menampung barang.'
                    
                    in_warehouse = int(Barang.now_in_warehouse_capacity(conn, id_warehouse)[0])
                    # print(f"Current capacity in warehouse: {in_warehouse}")
                    new_current_cap = in_warehouse + additional_stok

                Barang.update(conn, id_warehouse, id_barang, updates)
                conn.commit()
                return True, 'Barang berhasil diupdate'
            
            except Exception as e:
                    conn.rollback()
                    print(f"Exception occurred: {e}")
                    return False, str(e)
            finally:
                conn.close()
        return False, "Database connection failed"
        

    def search_barang(self, id_warehouse, part_name):
        conn = self.connect()
        if conn:
            results = Barang.search(conn, id_warehouse, part_name)
            return results
        
    def lihat_barang(self, id_warehouse, id_barang):
        conn = self.connect()
        if conn:
            try:
                return Barang.find_by_id(conn, id_warehouse, id_barang)
            finally:
                conn.close()
        return None
    
    def lihat_1_status_pengiriman(self, id_warehouse, id_barang):
        conn = self.connect()
        if conn:
            try:
                cursor = conn.cursor()
                query = "SELECT status_pengiriman FROM barang WHERE id_warehouse = %s AND id_barang = %s"
                cursor.execute(query, (id_warehouse, id_barang,))
                result = cursor.fetchone()
                return result[0] if result else None
            finally:
                conn.close()
        return None
    

    def lihat_wh_status_pengiriman(self, id_warehouse):
        conn = self.connect()
        if conn:
            try:
                cursor = conn.cursor()
                query = "SELECT status_pengiriman FROM barang WHERE id_warehouse = %s"
                cursor.execute(query, (id_warehouse,))
                results = cursor.fetchall()
                for (status_pengiriman,) in results:  
                    if status_pengiriman == "Dalam Pengiriman":
                        return False  
                return True
            except Exception as e:
                print(f"An error occurred: {e}") 
                return None 
            finally:
                conn.close()
        else:
            return None 


    def pindah_barang(self, id_barang, curr_warehouse, dest_warehouse):
        conn = self.connect()
        if conn:
            try:
                current_capacity = Barang.now_capacity(conn, dest_warehouse)
                max_capacity = Barang.max_capacity(conn, dest_warehouse)
                stok_barang = Barang.now_stok_barang(conn, curr_warehouse, id_barang)

                current_capacity = int(current_capacity[0]) if current_capacity and current_capacity[0] is not None else 0
                max_capacity = int(max_capacity[0]) if max_capacity and max_capacity[0] is not None else 0
                stok_barang = int(stok_barang[0]) if stok_barang and stok_barang[0] is not None else 0

                if current_capacity is not None and stok_barang is not None:
                    stock_being_moved = stok_barang
                    new_capacity = current_capacity + stock_being_moved

                    if new_capacity <= max_capacity:
                        Barang.pindah(conn, curr_warehouse, id_barang, dest_warehouse)
                        conn.commit()
                        return True, "Item is being moved to the new warehouse."
                    else:
                        return False, "Warehouse capacity is insufficient to accommodate the item."
                else:
                    return False, "Warehouse or item not found."
            except Error as e:
                conn.rollback()
                return False, f"Error: {e}"
            finally:
                conn.close()
        return False, "Database connection failed"

    def pindahkan_semua_barang(self, curr_warehouse, dest_warehouse):
        conn = self.connect()
        if conn:
            try:
                current_capacity = Barang.now_capacity(conn, dest_warehouse)
                max_capacity = Barang.max_capacity(conn, dest_warehouse)

                if current_capacity and current_capacity[0] is not None:
                    current_capacity = int(current_capacity[0])
                else:
                    current_capacity = 0

                max_capacity = int(max_capacity[0]) if max_capacity and max_capacity[0] is not None else 0  

                if self.lihat_wh_status_pengiriman(curr_warehouse) == False:
                    return False, "Ada barang yang sedang dalam pengiriman"
                else :
                    if current_capacity is not None:
                        total_stock_info = Barang.stok_all(conn, curr_warehouse)
                        # print(total_stock_info)

                        if total_stock_info:
                            stock_being_moved = int(total_stock_info[0]) if total_stock_info else 0
                            new_capacity = current_capacity + stock_being_moved
                            # print(new_capacity)

                            if new_capacity <= max_capacity:
                                Barang.pindah_semua(conn, curr_warehouse, dest_warehouse)
                                conn.commit()
                                return True, "All items are being moved to the new warehouse."
                            else:
                                return False, "Warehouse capacity is insufficient to accommodate all items."
                        else:
                            return False, "No items found in the current warehouse."
                    else:
                        return False, "Destination warehouse not found."
            except Error as e:
                conn.rollback()
                return False, f"Error: {e}"
            finally:
                conn.close()
        return False, "Database connection failed"