import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal
from classes.TambahWarehouseDisplay import TambahWarehouseDisplay
from classes.DataWarehouseDisplay import DataWarehouseDisplay
from classes.IconTextButton import IconTextButton



class DaftarWarehouseDisplay(QWidget):
    def __init__(self, parent_page=None, warehouse_controller=None, barang_controller=None):
        super().__init__()
        self.parent_page = parent_page
        self.warehouse_controller = warehouse_controller
        self.barang_controller = barang_controller
        self.initUI()
        self.load_warehouse()

    def initUI(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)
        
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.background.setPixmap(QPixmap('../img/assets/warehouse_list/black.png').scaled(self.size(), Qt.KeepAspectRatioByExpanding))
        self.background.resize(self.width(), self.height())

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_list/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

        self.search_label = QLabel(self)
        self.search_label.setPixmap(QPixmap('../img/assets/warehouse_list/cari_warehouse.png').scaled(480, 62, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.search_label.move(475, 40)

        # Search bar
        self.search_input = QLineEdit(self)
        self.search_input.setFont(customFont)
        self.search_input.setPlaceholderText("Search warehouse...")
        self.search_input.move(550, 45)
        self.search_input.setFixedSize(400, 50)
        self.search_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)
        self.search_input.textChanged.connect(self.filter_warehouses)

        # Scroll Area
        self.scroll_area = QScrollArea(self)
        self.scroll_area.setGeometry(70, 150, 1240, 600)
        self.scroll_area.setStyleSheet("background: transparent; border: None;")
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setWidgetResizable(True)

        self.container = QWidget()
        self.vbox = QVBoxLayout(self.container)
        self.vbox.setAlignment(Qt.AlignHCenter | Qt.AlignTop)
        # self.container.setMinimumSize(self.scroll_area.width(), self.scroll_area.height())

        self.container.setLayout(self.vbox)
        self.scroll_area.setWidget(self.container)

        # add warehouse button
        self.add_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_list/add.png').scaled(35, 34, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.add_button.setIcon(QIcon(scaled_pixmap))
        self.add_button.setIconSize(scaled_pixmap.size())
        self.add_button.setFixedSize(scaled_pixmap.size())
        self.add_button.setStyleSheet("QPushButton { border: none; }")
        self.add_button.move(1340, 30)
        self.add_button.clicked.connect(self.open_add_warehouse)
    
    def open_warehouse_detail(self, warehouse_id):
        warehouse_data = self.warehouse_controller.lihat_warehouse(warehouse_id)
        if warehouse_data:
            self.warehouse_detail_page = DataWarehouseDisplay(warehouse_data, parent_page=self, warehouse_controller=self.warehouse_controller, barang_controller=self.barang_controller)
            self.warehouse_detail_page.show()
            self.close()
        else:
            QMessageBox.warning(self, "Error", "Warehouse not found.")
        
    def update_container_height(self):
        item_height = 85
        total_height = len(self.warehouse_buttons) * (item_height + 10)  
        self.container.setMinimumHeight(max(self.scroll_area.height(), total_height))

    def filter_warehouses(self, text):
        self.clear_layout(self.vbox)
        if text.strip() == '':  
            self.load_warehouse()
        else:
            results = self.warehouse_controller.search_warehouse(text)
            self.warehouse_buttons = []
            for result in results:
                new_button = IconTextButton('../img/assets/warehouse_list/warehouse.png', result['nama_warehouse'], self.container)
                new_button.clicked.connect(lambda _, warehouse_id=result['id_warehouse']: self.open_warehouse_detail(warehouse_id))
                self.vbox.addWidget(new_button)
                self.warehouse_buttons.append((result['nama_warehouse'], new_button))
            self.update_container_height()

    def clear_layout(self, layout):
        while layout.count():
            item = layout.takeAt(0)
            widget = item.widget()
            if widget is not None:
                widget.deleteLater()
            elif item.layout() is not None:
                self.clear_layout(item.layout())
        self.update_container_height()

    def load_warehouse(self):
        results = self.warehouse_controller.lihat_daftar_warehouse()
        self.warehouse_buttons = []
        for result in results:
            new_button = IconTextButton('../img/assets/warehouse_list/warehouse.png', result['nama_warehouse'], self.container)
            new_button.clicked.connect(lambda _, warehouse_id=result['id_warehouse']: self.open_warehouse_detail(warehouse_id))
            self.vbox.addWidget(new_button)
            self.warehouse_buttons.append((result['nama_warehouse'], new_button))
        self.update_container_height()

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
            if self.parent_page:
                self.parent_page.update_warehouse_count()
        self.close()

    def update_warehouse_list(self, new_warehouse_name):
        new_button = IconTextButton('../img/assets/warehouse_list/warehouse.png', new_warehouse_name, self.container)
        self.vbox.addWidget(new_button)
        self.warehouse_buttons.append((new_warehouse_name, new_button))

    def open_add_warehouse(self, warehouse=None):
        self.add_warehouse_page = TambahWarehouseDisplay(parent=self, warehouse_controller=self.warehouse_controller)
        self.add_warehouse_page.show()
        # self.hide()