import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal
from classes.PindahBarangDisplay import PindahBarangDisplay


class DataBarangDisplay(QWidget):
    def __init__(self, barang=None, parent_page=None, barang_controller=None, warehouse_controller=None):
        super().__init__()
        self.item_data = barang
        if isinstance(self.item_data['foto'], bytes):
            self.item_data['foto'] = self.item_data['foto'].decode('utf-8')
        self.parent_page = parent_page
        self.barang_controller = barang_controller
        self.warehouse_controller = warehouse_controller
        self.image_path = None
        self.initUI()
        self.installEventFilter(self)
        
    def initUI(self):
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.background.setPixmap(QPixmap('../img/assets/item_details/black.png').scaled(self.size(), Qt.KeepAspectRatioByExpanding))
        self.background.resize(self.width(), self.height())

        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        # nama barang text
        self.edit_profile_text = QLabel(self)
        self.edit_profile_text.setText(self.item_data['nama_barang'])
        self.edit_profile_text.setFont(customFont)
        self.edit_profile_text.setStyleSheet("color: white; font-size: 30px;")
        self.edit_profile_text.setAlignment(Qt.AlignHCenter)
        self.update_label_position()

        # box
        self.box_label = QLabel(self)
        self.box_label.setPixmap(QPixmap('../img/assets/item_details/info.png').scaled(600, 480, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.box_label.move(390, 250)

        # foto barang
        self.item_image = QLabel(self)
        foto_path = self.item_data['foto']
        if foto_path:
            if isinstance(foto_path, bytes):
                foto_path = foto_path.decode('utf-8')
            pixmap = QPixmap(foto_path).scaled(100, 100, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation)
        else:
            foto_path = '..//img//item_img//img_def.jpeg'
            pixmap = QPixmap(foto_path).scaled(100, 100, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation)

        # circle mask
        mask = QPixmap(100, 100)
        mask.fill(Qt.transparent)
        painter = QPainter(mask)
        painter.setRenderHint(QPainter.Antialiasing)
        path = QPainterPath()
        path.addEllipse(0, 0, 100, 100)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, pixmap)
        painter.end()

        self.item_image.setPixmap(mask)
        self.item_image.move(638, 70)
        self.edit_foto_button = QPushButton(self)

        pixmap = QPixmap('../img/assets/item_details/edit_gambar.png')
        scaled_pixmap = pixmap.scaled(40, 40, Qt.KeepAspectRatio, Qt.SmoothTransformation) 
        self.edit_foto_button.setIcon(QIcon(scaled_pixmap))
        self.edit_foto_button.setIconSize(scaled_pixmap.size())
        self.edit_foto_button.setFixedSize(40,40)
        self.edit_foto_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_foto_button.move(700, 140)
        self.edit_foto_button.clicked.connect(self.uploadImage)

        # id barang label
        self.id_text = QLabel(self)
        self.id_text.setText(self.item_data['id_barang'])
        self.id_text.setFont(customFont)
        self.id_text.move(680, 268)
        self.id_text.setStyleSheet("color: grey; font-size: 20px;")

        # nama item label
        self.nama_item_input = QLineEdit(self)
        self.nama_item_input.setText(self.item_data['nama_barang'])
        self.nama_item_input.setFont(customFont)
        self.nama_item_input.setFixedSize(600, 50)
        self.nama_item_input.move(410, 335)
        self.nama_item_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.nama_item_input)

        # edit nama item button
        self.edit_nama_item_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_nama_item_button.setIcon(QIcon(scaled_pixmap))
        self.edit_nama_item_button.setIconSize(scaled_pixmap.size())
        self.edit_nama_item_button.setFixedSize(scaled_pixmap.size())
        self.edit_nama_item_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_nama_item_button.move(945, 348)
        self.edit_nama_item_button.clicked.connect(lambda: self.toggle_edit(self.nama_item_input))

        # stok label
        self.stok_input = QLineEdit(self)
        self.stok_input.setText(str(self.item_data['stok'])) 
        self.stok_input.setFont(customFont)
        self.stok_input.setFixedSize(600, 50)
        self.stok_input.move(410, 415)
        self.stok_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.stok_input)

        # edit stok button
        self.edit_stok_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_stok_button.setIcon(QIcon(scaled_pixmap))
        self.edit_stok_button.setIconSize(scaled_pixmap.size())
        self.edit_stok_button.setFixedSize(scaled_pixmap.size())
        self.edit_stok_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_stok_button.move(945, 428)
        self.edit_stok_button.clicked.connect(lambda: self.toggle_edit(self.stok_input))

        # deskripsi label
        self.deskripsi_input = QLineEdit(self)
        self.deskripsi_input.setText(self.item_data['deskripsi_barang'])
        self.deskripsi_input.setFont(customFont)
        self.deskripsi_input.setFixedSize(508, 50)
        self.deskripsi_input.move(410, 495)
        self.deskripsi_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.deskripsi_input)

        # edit max capacity button
        self.edit_deskripsi_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_deskripsi_button.setIcon(QIcon(scaled_pixmap))
        self.edit_deskripsi_button.setIconSize(scaled_pixmap.size())
        self.edit_deskripsi_button.setFixedSize(scaled_pixmap.size())
        self.edit_deskripsi_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_deskripsi_button.move(945, 508)
        self.edit_deskripsi_button.clicked.connect(lambda: self.toggle_edit(self.deskripsi_input))

        self.status_dropdown = QComboBox(self)
        self.status_dropdown.setFont(customFont)
        self.status_dropdown.addItems(['Dalam Pengiriman', 'Berada di Warehouse'])
        self.status_dropdown.move(405, 608)
        self.status_dropdown.setFixedSize(560, 50)
        self.status_dropdown.setStyleSheet("""
            QComboBox {
                background-color: transparent;
                color: white;
                padding: 5px;
            }
            QComboBox::drop-down {
                width: 20px;
            }
            QComboBox QAbstractItemView {
                background-color: black;
                color: white;
                selection-background-color: gray;
                selection-color: black;
            }
        """)

        current_status = self.item_data.get('status_pengiriman', 'Berada di Warehouse')  # default to 'Berada di Warehouse' if not specified
        status_index = self.status_dropdown.findText(current_status)
        self.status_dropdown.setCurrentIndex(status_index)

        # hapus barang button
        self.hapus_barang_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/item_details/hapus.png').scaled(145, 38, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.hapus_barang_button.setIcon(QIcon(scaled_pixmap))
        self.hapus_barang_button.setIconSize(scaled_pixmap.size())
        self.hapus_barang_button.setFixedSize(scaled_pixmap.size())
        self.hapus_barang_button.setStyleSheet("QPushButton { border: none; }")
        self.hapus_barang_button.move(850, 700)
        self.hapus_barang_button.clicked.connect(self.confirm_delete)

        # simpan barang button
        self.simpan_barang_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/item_details/simpan.png').scaled(145, 38, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_barang_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_barang_button.setIconSize(scaled_pixmap.size())
        self.simpan_barang_button.setFixedSize(scaled_pixmap.size())
        self.simpan_barang_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_barang_button.move(390, 700)
        self.simpan_barang_button.clicked.connect(self.confirm_update)

        # pindah barang button
        self.pindah_barang_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/item_details/pindah.png').scaled(145, 38, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.pindah_barang_button.setIcon(QIcon(scaled_pixmap))
        self.pindah_barang_button.setIconSize(scaled_pixmap.size())
        self.pindah_barang_button.setFixedSize(scaled_pixmap.size())
        self.pindah_barang_button.setStyleSheet("QPushButton { border: none; }")
        self.pindah_barang_button.move(550, 700)
        self.pindah_barang_button.clicked.connect(self.open_move_barang)

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

    def setupEditableField(self, field):
        field.setStyleSheet("""
            QLineEdit {
                color: white;
                background: transparent;
                border: none;
            }
        """)
        field.setReadOnly(True)

    def eventFilter(self, obj, event):
        if event.type() == QEvent.MouseButtonPress:
            # check kalo clicknya diluar input field
            clicked_widget = self.childAt(event.pos())
            if not isinstance(clicked_widget, QLineEdit):
                # bikin smuanya readonly kalo di click diluar
                self.setAllInputsReadOnly()
            return False
        return super(DataBarangDisplay, self).eventFilter(obj, event)

    def setAllInputsReadOnly(self):
        self.nama_item_input.setReadOnly(True)
        self.stok_input.setReadOnly(True)
        self.deskripsi_input.setReadOnly(True)
        # self.status_input.setReadOnly(True)
        
    # handle edit
    def toggle_edit(self, field):
        field.setReadOnly(not field.isReadOnly())
        if field.isReadOnly():
            field.setStyleSheet("""
                QLineEdit {
                    color: white;
                    background: transparent;
                    border: none;
                }
            """)
        else:
            field.setFocus()
            field.setStyleSheet("""
                QLineEdit {
                    color: white;
                    background: transparent;
                    border: none;
                }
            """)

    def save_selection(self):
        selected_status = self.status_dropdown.currentText()
        # print(f"Selected status: {selected_status}")
        self.item_data['status_pengiriman'] = selected_status
    
    def confirm_update(self):
        reply = QMessageBox.question(self, 'Confirm Update', 'Are you sure you want to save changes?',
                                    QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.update_barang()

    def confirm_delete(self):
        reply = QMessageBox.question(self, 'Confirm Delete', 'Are you sure you want to delete this item?',
                                    QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.delete_barang()
            if self.parent_page:
                self.parent_page.close()
                self.parent_page.parent_page.open_warehouse_detail(self.parent_page.warehouse_data['id_warehouse'])


    def update_barang(self):
        initial_photo = self.item_data['foto']
        updates = self.item_data
        id_barang = self.item_data['id_barang']
        id_warehouse = self.item_data['id_warehouse']
        new_nama = self.nama_item_input.text()
        new_stok = self.stok_input.text()
        new_deskripsi = self.deskripsi_input.text()
        new_status = self.status_dropdown.currentText()
        new_foto = self.image_path

        if new_nama != self.item_data['nama_barang']:
            updates['nama_barang'] = new_nama
        if new_stok != self.item_data['stok']:
            updates['stok'] = new_stok
        if new_deskripsi != self.item_data['deskripsi_barang']:
            updates['deskripsi_barang'] = new_deskripsi
        if new_status != self.item_data['status_pengiriman']:
            updates['status_pengiriman'] = new_status
        if new_foto != None:
            updates['foto'] = new_foto

        if updates:
            success, message = self.barang_controller.update_barang(id_warehouse, id_barang, updates)
            if success:
                QMessageBox.information(self, 'Success', message)
                if (updates['foto'] == None):
                    updates['foto'] = initial_photo
                self.update_item_photo(updates['foto'])
                # Handle closing the current window and refreshing the parent page
                if self.parent_page:
                    self.close()
                    self.parent_page.parent_page.open_warehouse_detail(self.parent_page.warehouse_data["id_warehouse"])
            else:
                QMessageBox.critical(self, 'Error', message)
        
    def update_item_photo(self, new_foto):
        pixmap = QPixmap(new_foto).scaled(100, 100, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation)
        mask = QPixmap(100, 100)
        mask.fill(Qt.transparent)
        painter = QPainter(mask)
        painter.setRenderHint(QPainter.Antialiasing)
        path = QPainterPath()
        path.addEllipse(0, 0, 100, 100)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, pixmap)
        painter.end()
        self.item_image.setPixmap(mask)

    def delete_barang(self):
        barang_id = self.item_data['id_barang']
        warehouse_id = self.item_data['id_warehouse']
        self.barang_controller.hapus_barang(warehouse_id, barang_id) 
        QMessageBox.information(self, 'Success', 'Item successfully deleted.')
        self.close()

    def uploadImage(self):
        file_name, _ = QFileDialog.getOpenFileName(self, "Open Image", "", "Image Files (*.png *.jpg *jpeg *.bmp)")
        if file_name:
            image_folder = '..//img//item_img//'
            os.makedirs(image_folder, exist_ok=True)
            _, extension = os.path.splitext(file_name)
            new_file_name = os.path.join(image_folder, f'image_{uuid.uuid4().hex}{extension}')
            copyfile(file_name, new_file_name)
            self.image_path = new_file_name 


    def open_move_barang(self):
        self.move_item_page = PindahBarangDisplay(barang=self.item_data, parent=self, barang_controller=self.barang_controller, warehouse_controller=self.warehouse_controller)
        self.move_item_page.show()
        self.close()

    def update_label_position(self):
        label_width = self.edit_profile_text.sizeHint().width()
        parent_width = self.width() - 25
        center_x = (parent_width - label_width) // 2
        self.edit_profile_text.move(center_x, 180)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.update_label_position()

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()