import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal
from classes.TambahBarangDisplay import TambahBarangDisplay
from classes.IconTextButton import IconTextButton
from classes.IconTextButtonItem import IconTextButtonItem
from classes.DataBarangDisplay import DataBarangDisplay
from classes.PindahSemuaBarangDisplay import PindahSemuaBarangDisplay


class DataWarehouseDisplay(QWidget):
    def __init__(self, warehouse, parent_page=None, warehouse_controller=None, barang_controller=None):
        super().__init__()
        self.warehouse_data = warehouse
        self.parent_page = parent_page
        self.warehouse_controller = warehouse_controller
        self.barang_controller = barang_controller
        self.initUI()
        self.installEventFilter(self)
        self.load_items()
        
    def initUI(self):
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.background.setPixmap(QPixmap('../img/assets/warehouse_details/black.png').scaled(self.size(), Qt.KeepAspectRatioByExpanding))
        self.background.resize(self.width(), self.height())

        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        # edit profile text
        self.edit_profile_text = QLabel(self)
        self.edit_profile_text.setText(self.warehouse_data['nama_warehouse'])
        self.edit_profile_text.setFont(customFont)
        self.edit_profile_text.move(340, 70)
        self.edit_profile_text.setStyleSheet("color: white; font-size: 30px;")

        # box
        self.box_label = QLabel(self)
        self.box_label.setPixmap(QPixmap('../img/assets/warehouse_details/info.png').scaled(700, 540, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.box_label.move(215, 140)

        self.id_text = QLabel(self)
        self.id_text.setText(str(self.warehouse_data['id_warehouse']))  # Convert to string
        self.id_text.setFont(customFont)
        self.id_text.move(465, 165)
        self.id_text.setStyleSheet("color: grey; font-size: 20px;")

        # nama warehouse label
        self.nama_warehouse_input = QLineEdit(self)
        self.nama_warehouse_input.setText(self.warehouse_data['nama_warehouse'])
        self.nama_warehouse_input.setFont(customFont)
        self.nama_warehouse_input.setFixedSize(600, 50)
        self.nama_warehouse_input.move(240, 283)
        self.nama_warehouse_input.setStyleSheet("""
            QLineEdit {
                color: grey;
                background: transparent;
                border: none;
            }
            """)
        self.nama_warehouse_input.setReadOnly(True)

        # current capacity label
        self.current_capacity_input = QLineEdit(self)
        self.current_capacity_input.setText(str(self.warehouse_data['kapasitas_terkini']))  # Convert to string
        self.current_capacity_input.setFont(customFont)
        self.current_capacity_input.setFixedSize(600, 50)
        self.current_capacity_input.move(240, 473)
        self.current_capacity_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.current_capacity_input)

        # edit current capacity button
        self.edit_current_capacity_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_current_capacity_button.setIcon(QIcon(scaled_pixmap))
        self.edit_current_capacity_button.setIconSize(scaled_pixmap.size())
        self.edit_current_capacity_button.setFixedSize(scaled_pixmap.size())
        self.edit_current_capacity_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_current_capacity_button.move(670, 483)
        self.edit_current_capacity_button.clicked.connect(lambda: self.toggle_edit(self.current_capacity_input))

        # max capacity label
        self.max_capacity_input = QLineEdit(self)
        self.max_capacity_input.setText(str(self.warehouse_data['kapasitas_maksimum']))  # Convert to string
        self.max_capacity_input.setFont(customFont)
        self.max_capacity_input.setFixedSize(600, 50)
        self.max_capacity_input.move(240, 378)
        self.max_capacity_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.max_capacity_input)

        # edit max capacity button
        self.edit_max_capacity_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_max_capacity_button.setIcon(QIcon(scaled_pixmap))
        self.edit_max_capacity_button.setIconSize(scaled_pixmap.size())
        self.edit_max_capacity_button.setFixedSize(scaled_pixmap.size())
        self.edit_max_capacity_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_max_capacity_button.move(670, 388)
        self.edit_max_capacity_button.clicked.connect(lambda: self.toggle_edit(self.max_capacity_input))

        # alamat label
        self.alamat_input = QLineEdit(self)
        self.alamat_input.setText(self.warehouse_data['alamat'])
        self.alamat_input.setFont(customFont)
        self.alamat_input.setFixedSize(420, 50)
        self.alamat_input.move(240, 568)
        self.alamat_input.setStyleSheet("color: grey;")
        self.setupEditableField(self.alamat_input)

        # edit alamat button
        self.edit_alamat_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_alamat_button.setIcon(QIcon(scaled_pixmap))
        self.edit_alamat_button.setIconSize(scaled_pixmap.size())
        self.edit_alamat_button.setFixedSize(scaled_pixmap.size())
        self.edit_alamat_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_alamat_button.move(670, 578)
        self.edit_alamat_button.clicked.connect(lambda: self.toggle_edit(self.alamat_input))

        # hapus warehouse button
        self.hapus_warehouse_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/hapus.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.hapus_warehouse_button.setIcon(QIcon(scaled_pixmap))
        self.hapus_warehouse_button.setIconSize(scaled_pixmap.size())
        self.hapus_warehouse_button.setFixedSize(scaled_pixmap.size())
        self.hapus_warehouse_button.setStyleSheet("QPushButton { border: none; }")
        self.hapus_warehouse_button.move(550, 700)
        self.hapus_warehouse_button.clicked.connect(self.confirm_delete)

        # simpan warehouse button
        self.simpan_warehouse_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/simpan.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_warehouse_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_warehouse_button.setIconSize(scaled_pixmap.size())
        self.simpan_warehouse_button.setFixedSize(scaled_pixmap.size())
        self.simpan_warehouse_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_warehouse_button.move(215, 700)
        self.simpan_warehouse_button.clicked.connect(self.confirm_update)  # Connect to confirm_update method

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

        # border search barang
        self.border_label = QLabel(self)
        self.border_label.setPixmap(QPixmap('../img/assets/warehouse_details/border_barang.png').scaled(500, 708, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.border_label.move(867, 50)

        # pindahkan barang button
        self.pindahkan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/pindah_semua_barang.png').scaled(280, 40, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.pindahkan_button.setIcon(QIcon(scaled_pixmap))
        self.pindahkan_button.setIconSize(scaled_pixmap.size())
        self.pindahkan_button.setFixedSize(scaled_pixmap.size())
        self.pindahkan_button.setStyleSheet("QPushButton { border: none; }")
        self.pindahkan_button.move(955, 670)
        self.pindahkan_button.clicked.connect(self.open_move_all_item_page)

        # add barang button
        self.add_barang_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/add.png').scaled(35, 34, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.add_barang_button.setIcon(QIcon(scaled_pixmap))
        self.add_barang_button.setIconSize(scaled_pixmap.size())
        self.add_barang_button.setFixedSize(scaled_pixmap.size())
        self.add_barang_button.setStyleSheet("QPushButton { border: none; }")
        self.add_barang_button.move(890, 675)
        self.add_barang_button.clicked.connect(self.open_add_item_page)

        # barang-barang di warehouse
        self.search_label = QLabel(self)
        self.search_label.setPixmap(QPixmap('../img/assets/warehouse_details/cari_barang.png').scaled(390, 60, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.search_label.move(890, 80)

        self.search_input = QLineEdit(self)
        self.search_input.setFont(customFont)
        self.search_input.setPlaceholderText("Search item...")
        self.search_input.move(957, 83)
        self.search_input.setFixedSize(400, 50)
        self.search_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)
        self.search_input.textChanged.connect(self.filter_items)

        self.scroll_area = QScrollArea(self)
        self.scroll_area.setGeometry(840, 150, 500, 500)
        self.scroll_area.setStyleSheet("background: transparent; border: None;")
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setWidgetResizable(True)

        self.container = QWidget()
        self.vbox = QVBoxLayout(self.container)
        self.vbox.setAlignment(Qt.AlignHCenter | Qt.AlignTop)

        self.container.setLayout(self.vbox)
        self.scroll_area.setWidget(self.container)

    def update_container_height(self):
        item_height = 70
        total_height = len(self.item_buttons) * (item_height + 10)
        self.container.setMinimumHeight(max(self.scroll_area.height(), total_height))

    def filter_items(self, text):
        # self.clear_layout(self.vbox)
        if text.strip() == '':  
            # self.clear_layout(self.vbox)
            self.load_items()
        else:
            results = self.barang_controller.search_barang(self.warehouse_data['id_warehouse'], text)
            self.item_buttons = []
            for i in reversed(range(self.vbox.count())): 
                widget_to_remove = self.vbox.itemAt(i).widget()
                self.vbox.removeWidget(widget_to_remove)
                widget_to_remove.setParent(None)

            if results :
                for result in results:
                    button = IconTextButtonItem('../img/assets/warehouse_details/barang_icon.png', result['nama_barang'], self.container)
                    button.setFixedSize(450, 70)
                    button.clicked.connect(lambda _, warehouse_id=result['id_warehouse'], id_barang=result['id_barang']: self.open_item_detail(warehouse_id, id_barang))
                    self.vbox.addWidget(button)
                    self.item_buttons.append((result['nama_barang'], button))
                self.update_container_height()
        self.container.setLayout(self.vbox)
        self.scroll_area.setWidget(self.container)

    def clear_layout(self, layout):
        while layout.count():
            item = layout.takeAt(0)
            widget = item.widget()
            if widget is not None:
                widget.deleteLater()
            elif item.layout() is not None:
                self.clear_layout(item.layout())
        self.update_container_height()


    def load_items(self):
        warehouse_id = self.warehouse_data['id_warehouse']
        results = self.barang_controller.lihat_daftar_barang(warehouse_id)
        self.item_buttons = []

        for i in reversed(range(self.vbox.count())): 
            widget_to_remove = self.vbox.itemAt(i).widget()
            self.vbox.removeWidget(widget_to_remove)
            widget_to_remove.setParent(None)

        if results :
            for result in results:
                button = IconTextButtonItem('../img/assets/warehouse_details/barang_icon.png', result['nama_barang'], self.container)
                button.setFixedSize(450, 70)
                button.clicked.connect(lambda _, id_barang=result['id_barang']: self.open_item_detail(warehouse_id, id_barang))
                self.vbox.addWidget(button)
                self.item_buttons.append((result['nama_barang'], button))
            self.update_container_height()


    def setupEditableField(self, field):
        field.setStyleSheet("""
            QLineEdit {
                color: white;
                background: transparent;
                border: none;
            }
        """)
        field.setReadOnly(True)

    def eventFilter(self, obj, event):
        if event.type() == QEvent.MouseButtonPress:
            clicked_widget = self.childAt(event.pos())
            if not isinstance(clicked_widget, QLineEdit):
                self.setAllInputsReadOnly()
            return False
        return super(DataWarehouseDisplay, self).eventFilter(obj, event)

    def setAllInputsReadOnly(self):
        self.max_capacity_input.setReadOnly(True)
        self.current_capacity_input.setReadOnly(True)
        self.alamat_input.setReadOnly(True)
    
    # handle edit
    def toggle_edit(self, field):
        field.setReadOnly(not field.isReadOnly())
        if field.isReadOnly():
            field.setStyleSheet("""
                QLineEdit {
                    color: white;
                    background: transparent;
                    border: none;
                }
            """)
        else:
            field.setFocus()
            field.setStyleSheet("""
                QLineEdit {
                    color: white;
                    background: transparent;
                    border: none;
                }
            """)

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()

    def confirm_delete(self):
        reply = QMessageBox.question(self, 'Confirm Delete', 'Apakah anda yakin untuk menghapus warehouse ini?',
                                     QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.delete_warehouse()
            if self.parent_page:
                self.parent_page.search_input.setText("")
                self.parent_page.search_input.setPlaceholderText("Search warehouse...")
                self.parent_page.filter_warehouses('')
                self.parent_page.show()

    def delete_warehouse(self):
        warehouse_id = self.warehouse_data['id_warehouse']
        self.warehouse_controller.hapus_warehouse(warehouse_id) 
        self.goBack()

    def confirm_update(self):
        reply = QMessageBox.question(self, 'Confirm Update', 'Apakah anda yakin untuk melakukan perubahan pada warehouse ini?',
                                     QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.update_warehouse_data()

    def update_warehouse_data(self):
        warehouse_id = self.warehouse_data['id_warehouse']
        new_max_capacity = self.max_capacity_input.text().strip()
        new_current_capacity = self.current_capacity_input.text().strip()
        new_location = self.alamat_input.text().strip()

        if not new_max_capacity or not new_current_capacity or not new_location:
            QMessageBox.warning(self, "Error", "Kapasitas maksimum, kapasitas terkini, dan alamat harus diisi.")
            return

        updates = {}
        try:
            new_max_capacity = int(new_max_capacity)
            if new_max_capacity <= 0:
                QMessageBox.warning(self, "Error", "Kapasitas maksimum harus lebih dari 0.")
                return
            updates['kapasitas_maksimum'] = new_max_capacity
        except ValueError:
            QMessageBox.warning(self, "Error", "Kapasitas maksimum harus berupa angka yang valid.")
            return

        try:
            new_current_capacity = int(new_current_capacity)
            if new_current_capacity < 0:
                QMessageBox.warning(self, "Error", "Kapasitas terkini tidak boleh negatif.")
                return
            updates['kapasitas_terkini'] = new_current_capacity
        except ValueError:
            QMessageBox.warning(self, "Error", "Kapasitas terkini harus berupa angka yang valid.")
            return

        updates['alamat'] = new_location

        if updates:
            success, message = self.warehouse_controller.update_warehouse(warehouse_id, updates)
            if not success:
                QMessageBox.warning(self, "Error", message)


    def open_move_all_item_page(self):
        self.move_all_item_page = PindahSemuaBarangDisplay(self.warehouse_data['id_warehouse'], parent_page=self, barang_controller=self.barang_controller)
        self.move_all_item_page.show()
        self.close()

    def open_add_item_page(self):
        self.move_add_item_page = TambahBarangDisplay(self.warehouse_data['id_warehouse'], parent=self, barang_controller=self.barang_controller)
        self.move_add_item_page.show()

    def open_item_detail(self, warehouse_id, item_id):
        barang_data = self.barang_controller.lihat_barang(warehouse_id, item_id)
        if barang_data:
            self.barang_detail_page = DataBarangDisplay(barang_data, parent_page=self, barang_controller=self.barang_controller, warehouse_controller=self.warehouse_controller)
            self.barang_detail_page.show()
            self.close()
        else:
            QMessageBox.warning(self, "Error", "Barang tidak ditemukan.")

    def update_item_list(self, new_item):
        new_button = IconTextButton('../img/assets/warehouse_list/warehouse.png', new_item, self.container)
        self.vbox.addWidget(new_button)
        self.item_buttons.append((new_item, new_button))