import os
from PyQt5.QtWidgets import QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase
from PyQt5.QtCore import Qt, QEvent


class EditProfileDisplay(QWidget):
    def __init__(self, profile_data, admin_controller, parent_page=None):
        super().__init__()
        self.admin_controller = admin_controller
        self.profile_data = profile_data
        self.parent_page = parent_page
        self.initUI()
        self.installEventFilter(self)

    def initUI(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.pixmap = QPixmap('../img/assets/edit_profile/black.png')
        self.background.setPixmap(self.pixmap.scaled(1440, 1024, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation))
        self.background.resize(self.width(), self.height())

        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/back.png').scaled(56, 56, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

        self.edit_profile_text = QLabel(self)
        self.edit_profile_text.setText("Edit Profile")
        self.edit_profile_text.setFont(customFont)
        self.edit_profile_text.move(600, 145)
        self.edit_profile_text.setStyleSheet("color: white; font-size: 30px;")

        self.id_label = QLabel(self)
        self.id_label.setPixmap(QPixmap('../img/assets/edit_profile/rectangle.png').scaled(600, 73, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.id_label.move(580, 220)

        self.id_text = QLabel(self)
        self.id_text.setText("Warehosea")
        self.id_text.setFont(customFont)
        self.id_text.move(635, 243)
        self.id_text.setStyleSheet("color: grey; font-size: 20px;")
        
        self.nama_label = QLabel(self)
        self.nama_label.setPixmap(QPixmap('../img/assets/edit_profile/nama.png').scaled(600, 73, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.nama_label.move(380, 320)

        self.name_input = QLineEdit(self)
        self.name_input.setText(self.profile_data[1])
        self.name_input.setFont(customFont)
        self.name_input.setFixedSize(600, 50)
        self.name_input.move(400, 335)
        self.setupEditableField(self.name_input, editable=True)

        self.edit_nama_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_nama_button.setIcon(QIcon(scaled_pixmap))
        self.edit_nama_button.setIconSize(scaled_pixmap.size())
        self.edit_nama_button.setFixedSize(scaled_pixmap.size())
        self.edit_nama_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_nama_button.move(930, 345)
        self.edit_nama_button.clicked.connect(lambda: self.toggle_edit(self.name_input))

        self.surel_label = QLabel(self)
        self.surel_label.setPixmap(QPixmap('../img/assets/edit_profile/surel.png').scaled(600, 73, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.surel_label.move(380, 400)

        self.surel_input = QLineEdit(self)
        self.surel_input.setText(self.profile_data[0])
        self.surel_input.setFont(customFont)
        self.surel_input.setFixedSize(600, 50)
        self.surel_input.move(400, 415)
        self.setupEditableField(self.surel_input, editable=False, color='grey')  # Make sure it's not editable and grey

        self.no_telp_label = QLabel(self)
        self.no_telp_label.setPixmap(QPixmap('../img/assets/edit_profile/no_telp.png').scaled(600, 73, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.no_telp_label.move(380, 480)

        self.no_telp_input = QLineEdit(self)
        self.no_telp_input.setText(self.profile_data[3])
        self.no_telp_input.setFont(customFont)
        self.no_telp_input.setFixedSize(600, 50)
        self.no_telp_input.move(400, 495)
        self.setupEditableField(self.no_telp_input, editable=True)

        self.edit_no_telp_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_no_telp_button.setIcon(QIcon(scaled_pixmap))
        self.edit_no_telp_button.setIconSize(scaled_pixmap.size())
        self.edit_no_telp_button.setFixedSize(scaled_pixmap.size())
        self.edit_no_telp_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_no_telp_button.move(930, 505)
        self.edit_no_telp_button.clicked.connect(lambda: self.toggle_edit(self.no_telp_input))

        self.password_label = QLabel(self)
        self.password_label.setPixmap(QPixmap('../img/assets/edit_profile/password.png').scaled(600, 73, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.password_label.move(380, 560)

        self.password_input = QLineEdit(self)
        self.password_input.setText(self.profile_data[2])
        self.password_input.setFont(customFont)
        self.password_input.setFixedSize(600, 50)
        self.password_input.move(400, 575)
        self.setupEditableField(self.password_input, editable=True)

        self.edit_password_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/pencil.png').scaled(26, 26, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_password_button.setIcon(QIcon(scaled_pixmap))
        self.edit_password_button.setIconSize(scaled_pixmap.size())
        self.edit_password_button.setFixedSize(scaled_pixmap.size())
        self.edit_password_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_password_button.move(930, 585)
        self.edit_password_button.clicked.connect(lambda: self.toggle_edit(self.password_input))

        self.simpan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/edit_profile/simpan.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_button.setIconSize(scaled_pixmap.size())
        self.simpan_button.setFixedSize(scaled_pixmap.size())
        self.simpan_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_button.move(580, 670)
        self.simpan_button.clicked.connect(self.confirm_changes)

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()

    def setupEditableField(self, field, editable=True, color='white'):
        field.setStyleSheet(f"""
            QLineEdit {{
                color: {color};
                background: transparent;
                border: none;
            }}
        """)
        field.setReadOnly(not editable)

    def eventFilter(self, obj, event):
        if event.type() == QEvent.MouseButtonPress:
            clicked_widget = self.childAt(event.pos())
            if not isinstance(clicked_widget, QLineEdit):
                self.setAllInputsReadOnly()
            return False
        return super(EditProfileDisplay, self).eventFilter(obj, event)

    def setAllInputsReadOnly(self):
        self.name_input.setReadOnly(True)
        self.surel_input.setReadOnly(True)
        self.no_telp_input.setReadOnly(True)
        self.password_input.setReadOnly(True)
    
    def toggle_edit(self, field):
        if field is not self.surel_input:
            field.setReadOnly(not field.isReadOnly())
            if field.isReadOnly():
                field.setStyleSheet("""
                    QLineEdit {
                        color: white;
                        background: transparent;
                        border: none;
                    }
                """)
            else:
                field.setFocus()
                field.setStyleSheet("""
                    QLineEdit {
                        color: white;
                        background: transparent;
                        border: none;
                    }
                """)

    def confirm_changes(self):
        self.dialog = AdminConfirmationDialog(self)
        self.dialog.show()

    def save_changes(self):
        self.profile_data[1] = self.name_input.text()
        self.profile_data[3] = self.no_telp_input.text()
        self.profile_data[2] = self.password_input.text()

        if not self.profile_data[1] or not self.profile_data[3] or not self.profile_data[2]:
            QMessageBox.warning(self, "Error", "Nama, No. Telp, dan Password tidak boleh kosong.")
            return
        
        if self.parent_page:
            self.parent_page.update_profile_data(self.profile_data)
        
        success, message = self.admin_controller.update_profil(self.profile_data)
        if not success :
            QMessageBox.warning(self, "Error", message)

    def reject_changes(self):
        self.name_input.setText(self.profile_data[1])
        self.surel_input.setText(self.profile_data[0])
        self.no_telp_input.setText(self.profile_data[3])
        self.password_input.setText(self.profile_data[2])


class AdminConfirmationDialog(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent, Qt.Window)
        self.loadFont()
        self.setupUI()

    def loadFont(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        self.customFont = QFont(fontFamily, 20)

    def setupUI(self):
        self.setWindowTitle("Konfirmasi Perubahan")
        self.setFixedSize(600, 200)
        layout = QVBoxLayout()

        label = QLabel("Apakah yakin untuk melakukan perubahan data admin ini?")
        label.setAlignment(Qt.AlignCenter)
        label.setFont(self.customFont)
        layout.addWidget(label)

        btnLayout = QHBoxLayout()
        saveBtn = QPushButton("Ya, Simpan")
        saveBtn.setFont(self.customFont)
        saveBtn.clicked.connect(self.accept)
        cancelBtn = QPushButton("Batal")
        cancelBtn.setFont(self.customFont)
        cancelBtn.clicked.connect(self.reject)

        btnLayout.addWidget(saveBtn)
        btnLayout.addWidget(cancelBtn)
        layout.addLayout(btnLayout)

        self.setLayout(layout)

    def accept(self):
        self.parent().save_changes()
        self.close()

    def reject(self):
        self.parent().reject_changes()
        self.close()