import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal
from classes.WarehouseController import WarehouseController
from classes.AdminController import AdminController
from classes.BarangController import BarangController
from classes.EditProfileDisplay import EditProfileDisplay
from classes.DaftarWarehouseDisplay import DaftarWarehouseDisplay



class HomeDisplay(QWidget):
    def __init__(self, name):
        super().__init__()
        self.name = name
        self.warehouse_controller = WarehouseController()
        self.warehouse_num = self.warehouse_controller.total_warehouse()
        self.initUI()

    def initUI(self):
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.pixmap = QPixmap('../img/assets/page3/black.png')
        self.background.setPixmap(self.pixmap.scaled(1440, 1024, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation))
        self.background.resize(self.width(), self.height())

        self.edit_profile_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/page3/Hosea.png').scaled(200, 150, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.edit_profile_button.setIcon(QIcon(scaled_pixmap))
        self.edit_profile_button.setIconSize(scaled_pixmap.size())
        self.edit_profile_button.setFixedSize(scaled_pixmap.size())
        self.edit_profile_button.setStyleSheet("QPushButton { border: none; }")
        self.edit_profile_button.move(470, 180)
        self.edit_profile_button.clicked.connect(self.open_edit_profile)

        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        self.hi_label = QLabel(self)
        self.hi_label.setText("Hi,")
        self.hi_label.setFont(customFont)
        self.hi_label.setStyleSheet("color: white; font-size: 40px;")
        self.hi_label.move(660, 200)

        self.name_label = QLabel(self)
        self.name_label.setText(self.name + "!")
        self.name_label.setFont(customFont)
        self.name_label.move(660, 250)
        self.name_label.setStyleSheet("color: white; font-size: 35px;")
        self.update_username_label()
        self.name_label.setFixedSize(500, 50)

        self.warehouse_label = QLabel(self)
        self.warehouse_label.setPixmap(QPixmap('../img/assets/page3/daftar_warehouse.png').scaled(540, 93, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.warehouse_label.move(475, 365)

        self.warehouse_count = QLabel(self)
        self.warehouse_count.setText(str(self.warehouse_num) + " warehouse")
        self.warehouse_count.setFont(customFont)
        self.warehouse_count.move(582, 415)
        self.warehouse_count.setStyleSheet("color: white; font-size: 20px;")

        self.daftar_warehouse_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/page3/click.png').scaled(33, 33, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.daftar_warehouse_button.setIcon(QIcon(scaled_pixmap))
        self.daftar_warehouse_button.setIconSize(scaled_pixmap.size())
        self.daftar_warehouse_button.setFixedSize(scaled_pixmap.size())
        self.daftar_warehouse_button.setStyleSheet("QPushButton { border: none; }")
        self.daftar_warehouse_button.move(850, 395)
        self.daftar_warehouse_button.clicked.connect(self.open_warehouse_list)

    def update_username_label(self):
        self.name_label.setText(self.name + "!")

    def update_profile_data(self, new_data):
        self.email = new_data[0]
        self.name = new_data[1]
        self.password = new_data[2]
        self.notelp = new_data[3]
        self.name_label.setText(self.name + "!")

    def open_edit_profile(self):
        admin_controller = AdminController()
        new_data = admin_controller.lihat_profil()
        self.edit_profile_page = EditProfileDisplay(profile_data=new_data, admin_controller=admin_controller, parent_page=self)
        self.edit_profile_page.show()
        self.close()

    def open_warehouse_list(self):
        warehouse_controller = WarehouseController()
        barang_controller = BarangController()
        self.warehouse_list_page = DaftarWarehouseDisplay(parent_page=self, warehouse_controller=warehouse_controller, barang_controller=barang_controller)
        self.warehouse_list_page.show()
        self.hide()

    def update_warehouse_count(self):
        self.warehouse_num = self.warehouse_controller.total_warehouse()
        self.warehouse_count.setText(str(self.warehouse_num) + " warehouse")