import os
from PyQt5.QtWidgets import QWidget, QLabel, QPushButton, QLineEdit, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase
from PyQt5.QtCore import Qt
from classes.HomeDisplay import HomeDisplay



class LoginDisplay(QWidget):
    def __init__(self, admin_controller):
        super().__init__()
        self.admin_controller = admin_controller
        self.initUI()
    
    def initUI(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        self.setFixedSize(1400, 800)

        self.background = QLabel(self)
        self.pixmap = QPixmap('../img/assets/page2/black.png')
        self.background.setPixmap(self.pixmap.scaled(1440, 1024, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation))
        self.background.resize(self.width(), self.height())

        self.login_label = QLabel(self)
        self.login_label.setPixmap(QPixmap('../img/assets/page2/login_title.png').scaled(569, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.login_label.move(450, 150)

        self.login_label = QLabel(self)
        self.login_label.setPixmap(QPixmap('../img/assets/page2/username_input.png').scaled(700, 120, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.login_label.move(380, 280)

        self.email_input = QLineEdit(self)
        self.email_input.setFont(customFont)
        self.email_input.setPlaceholderText("Username")
        self.email_input.move(400, 320)
        self.email_input.setFixedSize(400, 50)
        self.email_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.login_label = QLabel(self)
        self.login_label.setPixmap(QPixmap('../img/assets/page2/password_input.png').scaled(700, 120, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.login_label.move(380, 430)

        self.password_input = QLineEdit(self)
        self.password_input.setFont(customFont)
        self.password_input.setEchoMode(QLineEdit.Password)
        self.password_input.setPlaceholderText("Password")
        self.password_input.move(400, 470)
        self.password_input.setFixedSize(400, 50)
        self.password_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.login_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/page2/login_button.png').scaled(348, 78, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.login_button.setIcon(QIcon(scaled_pixmap))
        self.login_button.setIconSize(scaled_pixmap.size())
        self.login_button.setFixedSize(scaled_pixmap.size())
        self.login_button.setStyleSheet("QPushButton { border: none; }")
        self.login_button.move(560, 580)
        self.login_button.clicked.connect(self.open_main_page)

        self.setWindowTitle('Login Page')

    def open_main_page(self):
        email = self.email_input.text()
        password = self.password_input.text()

        if self.admin_controller.validate_admin(email, password):
            self.profile_data = self.admin_controller.lihat_profil()
            self.main_page = HomeDisplay(self.profile_data[1])
            self.main_page.show()
            self.close()
        else:
            QMessageBox.warning(self, 'Login Failed', 'Invalid username or password')
