import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal


class PindahBarangDisplay(QWidget):
    def __init__(self, barang=None, parent=None, barang_controller=None, warehouse_controller=None):
        super().__init__()
        self.item_data = barang
        self.parent_page = parent
        self.barang_controller = barang_controller
        self.warehouse_controller = warehouse_controller
        self.warehouse_list = self.get_warehouse_list()
        self.initUI()
    
    def initUI(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        # window size
        self.setFixedSize(1400, 800)

        # bg img
        self.background = QLabel(self)
        self.pixmap = QPixmap('../img/assets/move_item/black.png')
        self.background.setPixmap(self.pixmap.scaled(1440, 1024, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation))
        self.background.resize(self.width(), self.height())

        # pindah barang title
        self.title_label = QLabel(self)
        self.title_label.setPixmap(QPixmap('../img/assets/move_item/title.png').scaled(569, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.title_label.move(410, 80)

        # template
        self.template_label = QLabel(self)
        self.template_label.setPixmap(QPixmap('../img/assets/move_item/template.png').scaled(790, 500, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.template_label.move(320, 180)

        # warehouse selection dropdown
        self.warehouse_dropdown = QComboBox(self)
        self.warehouse_dropdown.setFont(customFont)
        self.warehouse_dropdown.addItems([name for _, name in self.warehouse_list])
        self.warehouse_dropdown.move(365, 367)
        self.warehouse_dropdown.setFixedSize(640, 50)
        self.warehouse_dropdown.setStyleSheet("""
            QComboBox {
                background-color: transparent;
                color: white;
                padding: 5px;
            }
            QComboBox::drop-down {
                width: 20px;
            }
            QComboBox QAbstractItemView {
                background-color: black;
                color: white;
                selection-background-color: gray;
                selection-color: black;
            }
        """)

        # simpan button
        self.simpan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/move_item/simpan.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_button.setIconSize(scaled_pixmap.size())
        self.simpan_button.setFixedSize(scaled_pixmap.size())
        self.simpan_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_button.move(600, 710)
        self.simpan_button.clicked.connect(self.confirm_move)

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

    def get_warehouse_list(self):
        warehouses = self.warehouse_controller.lihat_daftar_warehouse()
        if warehouses:
            return [(warehouse['id_warehouse'], warehouse['nama_warehouse']) for warehouse in warehouses]
        return []
    
    def confirm_move(self):
        reply = QMessageBox.question(self, 'Confirm Movement', 'Are you sure you want to move this item?',
                                    QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.move_barang()

    def move_barang(self):
        selected_warehouse_index = self.warehouse_dropdown.currentIndex()
        dest_warehouse_id = self.warehouse_list[selected_warehouse_index][0]
        curr_warehouse_id = self.item_data['id_warehouse']
        id_barang = self.item_data['id_barang']

        if dest_warehouse_id == curr_warehouse_id:
            QMessageBox.warning(self, 'Error', 'Tidak bisa memindahkan ke warehouse yang sama.')
            return

        if self.barang_controller.lihat_1_status_pengiriman(curr_warehouse_id, id_barang) == 'Dalam Pengiriman':
            QMessageBox.warning(self, 'Error', 'Tidak bisa memindahkan barang yang sedang dalam pengiriman.')
            return

        success, message = self.barang_controller.pindah_barang(id_barang, curr_warehouse_id, dest_warehouse_id)
        if success:
            QMessageBox.information(self, 'Success', message)
            if self.parent_page:
                self.close()
                self.parent_page.parent_page.parent_page.open_warehouse_detail(self.parent_page.parent_page.warehouse_data['id_warehouse'])
        else:
            QMessageBox.critical(self, 'Error', message)
        

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()
