import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal
from classes.WarehouseController import WarehouseController


class PindahSemuaBarangDisplay(QWidget):
    def __init__(self, warehouse_id, parent_page=None, barang_controller=None):
        super().__init__()
        self.prev_warehouse = warehouse_id
        self.parent_page = parent_page
        self.barang_controller = barang_controller
        self.warehouse_list = []
        self.warehouse_map = {}
        self.get_warehouse_names()
        self.initUI()

    def get_warehouse_names(self):
        warehouse_controller = WarehouseController()
        results = warehouse_controller.lihat_daftar_warehouse()
        for result in results:
            self.warehouse_list.append(result['nama_warehouse'])
            self.warehouse_map[result['nama_warehouse']] = result['id_warehouse']
  
    def initUI(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.pixmap = QPixmap('../img/assets/move_item/black.png')
        self.background.setPixmap(self.pixmap.scaled(1440, 1024, Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation))
        self.background.resize(self.width(), self.height())

        self.title_label = QLabel(self)
        self.title_label.setPixmap(QPixmap('../img/assets/move_item/title.png').scaled(569, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.title_label.move(410, 80)

        self.template_label = QLabel(self)
        self.template_label.setPixmap(QPixmap('../img/assets/move_item/template.png').scaled(790, 500, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.template_label.move(320, 180)

        self.warehouse_dropdown = QComboBox(self)
        self.warehouse_dropdown.setFont(customFont)
        self.warehouse_dropdown.addItems(self.warehouse_list)
        self.warehouse_dropdown.move(365, 367)
        self.warehouse_dropdown.setFixedSize(640, 50)
        self.warehouse_dropdown.setStyleSheet("""
            QComboBox {
                background-color: transparent;
                color: white;
                padding: 5px;
            }
            QComboBox::drop-down {
                width: 20px;
            }
            QComboBox QAbstractItemView {
                background-color: black;
                color: white;
                selection-background-color: gray;
                selection-color: black;
            }
        """)

        # simpan button
        self.simpan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/move_item/simpan.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_button.setIconSize(scaled_pixmap.size())
        self.simpan_button.setFixedSize(scaled_pixmap.size())
        self.simpan_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_button.move(600, 710)
        self.simpan_button.clicked.connect(self.save_selection)

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

    def save_selection(self):
        selected_warehouse = self.warehouse_dropdown.currentText()
        selected_id_warehouse = self.warehouse_map[selected_warehouse]
        self.move_all_item_to_warehouse(selected_id_warehouse)

    def move_all_item_to_warehouse(self, selected_warehouse):\

        if selected_warehouse == self.prev_warehouse:
            QMessageBox.warning(self, "Error", "Tidak bisa memindahkan ke warehouse yang sama.")
            return
        
        success, message = self.barang_controller.pindahkan_semua_barang(self.prev_warehouse, selected_warehouse)
        
        if success:
            QMessageBox.information(self, "Success", message)
            if self.parent_page:
                self.close()
                self.parent_page.load_items()
                self.parent_page.parent_page.open_warehouse_detail(self.parent_page.warehouse_data['id_warehouse'])
        else:
            QMessageBox.critical(self, "Error", message)
        
    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()