import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal


class TambahBarangDisplay(QWidget):
    def __init__(self, warehouse_id, parent=None, barang_controller=None):
        super().__init__(parent)
        self.parent_page = parent
        self.id_warehouse = warehouse_id
        self.barang_controller = barang_controller
        self.image_path = None
        self.initUI()

    def initUI(self):
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.background.setPixmap(QPixmap('../img/assets/add_item/black.png').scaled(self.size(), Qt.KeepAspectRatioByExpanding))
        self.background.resize(self.width(), self.height())

        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        # add title
        self.add_warehouse_label = QLabel(self)
        self.add_warehouse_label.setPixmap(QPixmap('../img/assets/add_item/title.png').scaled(569, 55, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.add_warehouse_label.move(400, 60)

        # template
        self.template_label = QLabel(self)
        self.template_label.setPixmap(QPixmap('../img/assets/add_item/data.png').scaled(790, 500, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.template_label.move(320, 160)


        self.id_input = QLineEdit(self)
        self.id_input.setFont(customFont)
        self.id_input.setPlaceholderText("Masukkin ID")
        self.id_input.move(795, 172)
        self.id_input.setFixedSize(300, 50)
        self.id_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)
        self.nama_barang_input = QLineEdit(self)
        self.nama_barang_input.setFont(customFont)
        self.nama_barang_input.setPlaceholderText("Barang XX")
        self.nama_barang_input.move(375, 295)
        self.nama_barang_input.setFixedSize(300, 50)
        self.nama_barang_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.stok_input = QLineEdit(self)
        self.stok_input.setFont(customFont)
        self.stok_input.setPlaceholderText("Masukin angka ya!")
        self.stok_input.move(375, 385)
        self.stok_input.setFixedSize(300, 50)
        self.stok_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.deskripsi_input = QLineEdit(self)
        self.deskripsi_input.setFont(customFont)
        self.deskripsi_input.setPlaceholderText("Bisa isi warna, bentuk, dll :)")
        self.deskripsi_input.move(720, 295)
        self.deskripsi_input.setFixedSize(300, 50)
        self.deskripsi_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.status_input = QLineEdit(self)
        self.status_input.setFont(customFont)
        self.status_input.setText("Berada di Warehouse")
        self.status_input.setReadOnly(True)
        self.status_input.move(720, 470)
        self.status_input.setFixedSize(300, 50)
        self.status_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        # upload foto button
        self.upload_foto_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/add_item/upload_file.png').scaled(186, 58, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.upload_foto_button.setIcon(QIcon(scaled_pixmap))
        self.upload_foto_button.setIconSize(scaled_pixmap.size())
        self.upload_foto_button.setFixedSize(scaled_pixmap.size())
        self.upload_foto_button.setStyleSheet("QPushButton { border: none; }")
        self.upload_foto_button.move(375, 490)
        self.upload_foto_button.clicked.connect(self.uploadImage)

        # simpan button
        self.simpan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/add_item/simpanzee.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_button.setIconSize(scaled_pixmap.size())
        self.simpan_button.setFixedSize(scaled_pixmap.size())
        self.simpan_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_button.move(600, 690)
        self.simpan_button.clicked.connect(self.confirm_save)

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)

    def uploadImage(self):
        file_name, _ = QFileDialog.getOpenFileName(self, "Open Image", "", "Image Files (*.png *.jpg *.bmp)")
        if file_name:
            image_folder = 'img\\item_img'
            os.makedirs(image_folder, exist_ok=True)

            _, extension = os.path.splitext(file_name)
            new_file_name = os.path.join(image_folder, f'image_{uuid.uuid4().hex}{extension}')
            copyfile(file_name, new_file_name)

            self.image_path = new_file_name

    def confirm_save(self):
        id_item = self.id_input.text()
        name = self.nama_barang_input.text()
        stok = self.stok_input.text()
        deskripsi = self.deskripsi_input.text()
        foto = self.image_path

        if not (id_item and name and stok): 
            QMessageBox.warning(self, "Error", "ID, nama, dan stok perlu diisi.")
            return  # Exit the method if any field is empty

        
        if len(id_item) > 5:
            QMessageBox.warning(self, "Error", "ID barang maksimal 5 karakter.")
            return
        
        try:
            stok = int(stok)
            if stok > 0:  # Check if stok is positive after ensuring it's a valid integer
                self.dialog = AddItemConfirmationDialog(self)
                self.dialog.show()
            else:
                QMessageBox.warning(self, "Error", "Stok harus lebih besar dari 0.")
        except ValueError:
            QMessageBox.warning(self, "Error", "Stok harus berupa angka.")


    def save_changes(self):
        id_item = self.id_input.text()
        new_item_name = self.nama_barang_input.text()
        stok = int(self.stok_input.text())
        deskripsi = self.deskripsi_input.text()
        foto = self.image_path

        success, message = self.barang_controller.tambah_barang(self.id_warehouse, id_item, new_item_name, stok, deskripsi, foto)
        if not success:
            QMessageBox.warning(self, "Error", message)
        else:
            if self.parent_page:
                self.parent_page.close()
                self.parent_page.parent_page.open_warehouse_detail(self.id_warehouse)

    def reject_changes(self):
        # Tidak melakukan apa-apa
        self.close()

    def goBack(self):
        if self.parent_page:
            self.parent_page.show()
        self.close()

    

class AddItemConfirmationDialog(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent, Qt.Window)
        self.loadFont()
        self.setupUI()

    def loadFont(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        self.customFont = QFont(fontFamily, 20)

    def setupUI(self):
        self.setWindowTitle("Konfirmasi Penambahan Barang")
        self.setFixedSize(600, 200)
        layout = QVBoxLayout()

        label = QLabel("Apakah yakin untuk menambah barang ini ke warehouse?")
        label.setAlignment(Qt.AlignCenter)
        label.setFont(self.customFont)
        layout.addWidget(label)

        btnLayout = QHBoxLayout()
        saveBtn = QPushButton("Ya, tambah")
        saveBtn.setFont(self.customFont)
        saveBtn.clicked.connect(self.accept)
        cancelBtn = QPushButton("Batal")
        cancelBtn.setFont(self.customFont)
        cancelBtn.clicked.connect(self.reject)

        btnLayout.addWidget(saveBtn)
        btnLayout.addWidget(cancelBtn)
        layout.addLayout(btnLayout)

        self.setLayout(layout)

    def accept(self):
        self.parent().save_changes()
        self.close()

    def reject(self):
        self.parent().reject_changes()
        self.close()