import sys, os
import uuid
from shutil import copyfile
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QLineEdit, QVBoxLayout, QHBoxLayout, QScrollArea, QFileDialog, QComboBox, QMessageBox
from PyQt5.QtGui import QPixmap, QIcon, QFont, QFontDatabase, QPainter, QPainterPath
from PyQt5.QtCore import Qt, QEvent, pyqtSignal


class TambahWarehouseDisplay(QWidget):
    def __init__(self, parent=None, warehouse_controller=None):
        super().__init__(parent)
        self.parent_page = parent
        self.warehouse_controller = warehouse_controller
        self.initUI()

    def initUI(self):
        self.setFixedSize(1400, 800)
        self.background = QLabel(self)
        self.background.setPixmap(QPixmap('../img/assets/warehouse_list/black.png').scaled(self.size(), Qt.KeepAspectRatioByExpanding))
        self.background.resize(self.width(), self.height())

        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        customFont = QFont(fontFamily, 20)

        # add warehouse label
        self.add_warehouse_label = QLabel(self)
        self.add_warehouse_label.setPixmap(QPixmap('../img/assets/add_warehouse/tambah_warehouse.png').scaled(569, 55, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.add_warehouse_label.move(430, 60)

        # load template
        self.template_label = QLabel(self)
        self.template_label.setPixmap(QPixmap('../img/assets/add_warehouse/template.png').scaled(790, 500, Qt.KeepAspectRatio, Qt.SmoothTransformation))
        self.template_label.move(350, 160)

        self.nama_input = QLineEdit(self)
        self.nama_input.setFont(customFont)
        self.nama_input.setPlaceholderText("Warehouse XX")
        self.nama_input.move(425, 280)
        self.nama_input.setFixedSize(400, 50)
        self.nama_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.kapasitas_input = QLineEdit(self)
        self.kapasitas_input.setFont(customFont)
        self.kapasitas_input.setPlaceholderText("Masukin angka ya!")
        self.kapasitas_input.move(425, 375)
        self.kapasitas_input.setFixedSize(400, 50)
        self.kapasitas_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        self.alamat_input = QLineEdit(self)
        self.alamat_input.setFont(customFont)
        self.alamat_input.setPlaceholderText("Lokasi warehousemu ya, bukan pacarmu :)")
        self.alamat_input.move(425, 470)
        self.alamat_input.setFixedSize(400, 50)
        self.alamat_input.setStyleSheet("""
            QLineEdit {
                color: white;
                font-size: 20px;
                background: transparent;
                border: none;
            }
            QLineEdit::placeholder {
                color: white;
            }
        """)

        # simpan button
        self.simpan_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/add_warehouse/simpan.png').scaled(175, 48, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.simpan_button.setIcon(QIcon(scaled_pixmap))
        self.simpan_button.setIconSize(scaled_pixmap.size())
        self.simpan_button.setFixedSize(scaled_pixmap.size())
        self.simpan_button.setStyleSheet("QPushButton { border: none; }")
        self.simpan_button.move(600, 690)
        self.simpan_button.clicked.connect(self.confirm_save)

        # back button
        self.back_button = QPushButton(self)
        scaled_pixmap = QPixmap('../img/assets/warehouse_details/back.png').scaled(30, 30, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.back_button.setIcon(QIcon(scaled_pixmap))
        self.back_button.setIconSize(scaled_pixmap.size())
        self.back_button.setFixedSize(scaled_pixmap.size())
        self.back_button.setStyleSheet("QPushButton { border: none; }")
        self.back_button.move(30, 30)
        self.back_button.clicked.connect(self.goBack)


    def confirm_save(self):
        name = self.nama_input.text()
        max_capacity = self.kapasitas_input.text()
        location = self.alamat_input.text()
        
        if not name or not max_capacity or not location:
            QMessageBox.warning(self, "Error", "Nama, kapasitas maksimum, dan alamat perlu diisi.")
            return
        try:
            max_capacity = int(max_capacity)
            if name and max_capacity > 0 and location:
                self.dialog = AddWarehouseConfirmationDialog(self)
                self.dialog.show()
            elif max_capacity <= 0:
                QMessageBox.warning(self, "Error", "Kapasitas maksimum harus lebih dari 0.")
            else:
                QMessageBox.warning(self, "Error", "Nama, kapasitas maksimum, dan alamat perlu diisi.")
        except ValueError:
            QMessageBox.warning(self, "Error", "Kapasitas maksimum harus berupa angka.")

    def save_changes(self):
        new_warehouse_name = self.nama_input.text()
        max_capacity = int(self.kapasitas_input.text())
        location = self.alamat_input.text()

        success, message = self.warehouse_controller.tambah_warehouse(new_warehouse_name, max_capacity, location)
        if not success:
            QMessageBox.warning(self, "Error", message)
        else:
            if self.parent_page:
                self.parent_page.search_input.setText("")
                self.parent_page.search_input.setPlaceholderText("Search warehouse...")
                self.parent_page.filter_warehouses('')
                self.parent_page.show()
            self.close()

    def reject_changes(self):
        # Tidak melakukan apa-apa
        self.close()

    def goBack(self):
        self.parent_page.show()
        self.close()
        

class AddWarehouseConfirmationDialog(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent, Qt.Window)
        self.loadFont()
        self.setupUI()

    def loadFont(self):
        fontPath = os.path.abspath("../img/assets/Nunito.ttf")
        fontId = QFontDatabase.addApplicationFont(fontPath)
        fontFamily = QFontDatabase.applicationFontFamilies(fontId)[0]
        self.customFont = QFont(fontFamily, 20)

    def setupUI(self):
        self.setWindowTitle("Konfirmasi Penambahan Warehouse")
        self.setFixedSize(600, 200)
        layout = QVBoxLayout()

        label = QLabel("Apakah yakin untuk menambah warehouse ini?")
        label.setAlignment(Qt.AlignCenter)
        label.setFont(self.customFont)
        layout.addWidget(label)

        btnLayout = QHBoxLayout()
        self.saveBtn = QPushButton("Ya, tambah")
        self.saveBtn.setFont(self.customFont)
        self.saveBtn.clicked.connect(self.accept)
        self.cancelBtn = QPushButton("Batal")
        self.cancelBtn.setFont(self.customFont)
        self.cancelBtn.clicked.connect(self.reject)

        btnLayout.addWidget(self.saveBtn)
        btnLayout.addWidget(self.cancelBtn)
        layout.addLayout(btnLayout)

        self.setLayout(layout)

    def accept(self):
        self.parent().save_changes()
        self.close()

    def reject(self):
        self.parent().reject_changes()
        self.close()