import mysql.connector
from mysql.connector import Error

class Warehouse:
    def __init__(self, connection, id_warehouse=None, nama_warehouse=None, kapasitas_maksimum=None, alamat=None, kapasitas_terkini=0):
        self.connection = connection
        self.__id_warehouse = id_warehouse
        self.__nama_warehouse = nama_warehouse
        self.__kapasitas_maksimum = kapasitas_maksimum
        self.__alamat = alamat
        self.__kapasitas_terkini = kapasitas_terkini

    def get_id_warehouse(self) -> int:
        return self.__id_warehouse
    
    def get_nama_warehouse(self) -> str:
        return self.__nama_warehouse
    
    def get_kapasitas_maksimum(self) -> int:
        return self.__kapasitas_maksimum
    
    def get_alamat(self) -> str:
        return self.__alamat
    
    def get_kapasitas_terkini(self) -> int:
        return self.__kapasitas_terkini
    
    def set_id_warehouse(self, id_warehouse: int) -> None:
        self.__id_warehouse = id_warehouse

    def set_nama_warehouse(self, nama_warehouse: str) -> None:
        self.__nama_warehouse = nama_warehouse

    def set_kapasitas_maksimum(self, kapasitas_maksimum: int) -> None:
        self.__kapasitas_maksimum = kapasitas_maksimum

    def set_alamat(self, alamat: str) -> None:
        self.__alamat = alamat

    def set_kapasitas_terkini(self, kapasitas_terkini: int) -> None:
        self.__kapasitas_terkini = kapasitas_terkini

    @classmethod
    def add(cls, connection, nama_warehouse, kapasitas_maksimum, alamat):
        cursor = connection.cursor()
        query = "INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)"
        values = (nama_warehouse, kapasitas_maksimum, alamat, 0, 'hoseaganteng@gmail.com')
        cursor.execute(query, values)
        connection.commit()
        cursor.close()
        return cls(connection, cursor.lastrowid, nama_warehouse, kapasitas_maksimum, alamat, 0)

    @classmethod
    def deleteBarangFromWH(cls, connection, id_warehouse):
        cursor = connection.cursor()
        query = "DELETE FROM barang WHERE id_warehouse = %s"
        cursor.execute(query, (id_warehouse,))
        connection.commit()
        cursor.close()

    @classmethod
    def deleteWH(cls, connection, id_warehouse):
        cursor = connection.cursor(dictionary=True)
        query = "DELETE FROM warehouse WHERE id_warehouse = %s"
        cursor.execute(query, (id_warehouse,))
        connection.commit()
        cursor.close()

    @classmethod
    def update(cls, connection, id_warehouse, updates):
        cursor = connection.cursor()
        query_parts = [f"{key} = %s" for key in updates.keys()]
        query = f"UPDATE warehouse SET {', '.join(query_parts)} WHERE id_warehouse = %s"
        values = tuple(updates.values()) + (id_warehouse,)
        cursor.execute(query, values)
        connection.commit()
        cursor.close()

    @classmethod
    def search(cls, connection, part_name):
        cursor = connection.cursor(dictionary=True)
        query = "SELECT * FROM warehouse WHERE nama_warehouse LIKE %s"
        values = ('%' + part_name + '%',)
        cursor.execute(query, values)
        results = cursor.fetchall()
        cursor.close()
        return results

    @classmethod
    def find_by_id(cls, conn, id_warehouse):
        cursor = conn.cursor()
        query = "SELECT * FROM warehouse WHERE id_warehouse = %s"
        cursor.execute(query, (id_warehouse,))
        result = cursor.fetchone()
        
        if result:
            columns = [desc[0] for desc in cursor.description]
            warehouse_data = dict(zip(columns, result))
            return warehouse_data
        return None

    @classmethod
    def check_name(cls, connection, nama_warehouse) -> bool:
        cursor = connection.cursor()
        query = "SELECT COUNT(*) FROM warehouse WHERE nama_warehouse = %s"
        cursor.execute(query, (nama_warehouse,))
        if cursor.fetchone()[0] > 0:
            return True
        else :
            return False
        
    @classmethod
    def check_id(cls, connection, id_warehouse) -> bool:
        cursor = connection.cursor()
        query = "SELECT COUNT(*) FROM warehouse WHERE id_warehouse = %s"
        cursor.execute(query, (id_warehouse,))
        if cursor.fetchone()[0] > 0:
            return True
        else :
            return False
        
    @classmethod
    def get_nama_from_id(cls, connection, id_warehouse):
        cursor = connection.cursor()
        query = "SELECT nama_warehouse FROM warehouse WHERE id_warehouse = %s"
        cursor.execute(query, (id_warehouse,))
        result = cursor.fetchone()
        cursor.close()
        return result[0]
    

    @classmethod
    def get_num_warehouse(cls, connection):
        cursor = connection.cursor()
        cursor.execute("SELECT COUNT(*) FROM warehouse")
        result = cursor.fetchone()
        cursor.close()
        return result[0]