# tests/test_admin_controller.py

import sys
import os
import pytest
import mysql.connector

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(root_dir + '/src')

from classes.Admin import Admin
from classes.AdminController import AdminController

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def mock_admin():
    admin = Admin(surel="hoseaganteng@gmail.com", nama="Hosea Ganteng", password="password123", notelp="1234567890")
    return [admin]

@pytest.fixture
def setup_teardown_update(db_connection):
    cursor = db_connection.cursor()
    cursor.execute("SELECT surel, nama, password, noTelp FROM Admin WHERE surel = 'hoseaganteng@gmail.com'")
    original_data = cursor.fetchone()

    yield

    cursor.execute("UPDATE Admin SET nama = %s, password = %s, noTelp = %s WHERE surel = %s",
                   (original_data[1], original_data[2], original_data[3], original_data[0]))
    db_connection.commit()
    cursor.close()

@pytest.fixture
def controller(mock_admin):
    controller = AdminController()
    controller.admins = mock_admin
    return controller

def test_update_profil(controller, setup_teardown_update):
    profile_data = ["hoseaganteng@gmail.com", "Updated Name", "newpassword123", "1111111111"]
    controller.update_profil(profile_data)
    updated_admin = next(admin for admin in controller.admins if admin.get_surel() == "hoseaganteng@gmail.com")

    assert updated_admin.get_nama() == "Updated Name"
    assert updated_admin.get_password() == "newpassword123"
    assert updated_admin.get_no_telp() == "1111111111"

def test_validate_admin(controller):
    assert controller.validate_admin("hoseaganteng@gmail.com", "password123") == True
    assert controller.validate_admin("hoseaganteng@gmail.com", "wrongpassword") == False

def test_lihat_profil(controller):
    profile = controller.lihat_profil()

    assert profile[0] == "hoseaganteng@gmail.com"
    assert profile[1] == "Hosea Ganteng"
    assert profile[2] == "password123"
    assert profile[3] == "1234567890"

# Run the test
if __name__ == "__main__":
    pytest.main()
