import sys
import os
import pytest
import mysql.connector

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(root_dir + '/src/classes')

from src.classes.Barang import Barang

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def barang(db_connection):
    return Barang(conn=db_connection, id_barang='123AB', nama_barang="Item Test", stok=100, deskripsi="Test description", foto="test.jpg", status_pengiriman="Berada di Warehouse")

def test_get_id_barang(barang):
    assert barang.get_id_barang() == '123AB'

def test_get_nama_barang(barang):
    assert barang.get_nama_barang() == "Item Test"

def test_get_stok(barang):
    assert barang.get_stok() == 100

def test_get_deskripsi(barang):
    assert barang.get_deskripsi() == "Test description"

def test_get_foto(barang):
    assert barang.get_foto() == "test.jpg"

def test_get_status_pengiriman(barang):
    assert barang.get_status_pengiriman() == "Berada di Warehouse"

def test_set_nama_barang(barang):
    barang.set_nama_barang("New Item")
    assert barang.get_nama_barang() == "New Item"

def test_set_stok(barang):
    barang.set_stok(150)
    assert barang.get_stok() == 150

def test_set_deskripsi(barang):
    barang.set_deskripsi("Updated description")
    assert barang.get_deskripsi() == "Updated description"

def test_set_foto(barang):
    barang.set_foto("updated.jpg")
    assert barang.get_foto() == "updated.jpg"

def test_set_status_pengiriman(barang):
    barang.set_status_pengiriman("Dalam Pengiriman")
    assert barang.get_status_pengiriman() == "Dalam Pengiriman"

def test_add_barang(db_connection):
    Barang.add(conn=db_connection, id_warehouse=1, id_barang='123AB', nama_barang="New Item", deskripsi_barang="Another description", stok=50, foto="newitem.jpg")
    assert Barang.id_exist(conn=db_connection, id_barang='123AB')

def test_update_barang(db_connection):
    updates = {'nama_barang': "Updated Item", 'stok': 75}
    Barang.update(connection=db_connection, id_warehouse=1, id_barang='123AB', updates=updates)
    updated_data = Barang.find_by_id(conn=db_connection, id_warehouse=1, id_barang='123AB')
    assert updated_data['nama_barang'] == "Updated Item" and updated_data['stok'] == 75

def test_delete_barang(db_connection):
    Barang.delete(conn=db_connection, id_warehouse=1, id_barang='123AB')
    assert not Barang.id_exist(conn=db_connection, id_barang='123BC')

def test_search_barang(db_connection):
    results = Barang.search(conn=db_connection, id_warehouse=1, part_name="item")
    assert len(results) >= 0

def test_pindah_barang(db_connection):
    Barang.add(conn=db_connection, id_warehouse=1, id_barang='123AB', nama_barang="Test Item", deskripsi_barang="A test item", stok=10, foto="test.jpg")
    Barang.pindah(conn=db_connection, curr_warehouse=1, id_barang='123AB', dest_warehouse=2)
    data = Barang.find_by_id(conn=db_connection, id_warehouse=2, id_barang='123AB')
    assert data is not None and data['status_pengiriman'] == "Dalam Pengiriman"

if __name__ == "__main__":
    pytest.main()