import sys
import os
import pytest
import mysql.connector

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(root_dir + '/src/classes')

from src.classes.BarangController import BarangController
from src.classes.Barang import Barang

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def controller():
    return BarangController(**DB_CONFIG)


def test_lihat_daftar_barang(db_connection, controller):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, stok, deskripsi_barang, foto) VALUES (%s, %s, %s, %s, %s, %s)", 
                   (1, '123AB', "Test Barang 1", 100, "Description 1", 'ex1.jpg'))
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, stok, deskripsi_barang, foto) VALUES (%s, %s, %s, %s, %s, %s)", 
                   (1, '123AC', "Test Barang 2", 100, "Description 2", 'ex2.jpg'))
    db_connection.commit()
    results = controller.lihat_daftar_barang(id_warehouse=1)
    assert len(results) >= 2

    cursor.execute("DELETE FROM barang WHERE nama_barang IN (%s, %s)", ("Test Barang 1", "Test Barang 2"))
    db_connection.commit()
    cursor.close()

def test_tambah_barang(db_connection, controller):
    success, message = controller.tambah_barang(1, "ID123", "Test Barang", 50, "Test Deskripsi", "path/to/image.jpg")
    assert success == True
    assert message == "Barang berhasil ditambahkan."

    success, message = controller.tambah_barang(1, "ID123", "Test Barang", 50, "Test Deskripsi", "path/to/image.jpg")
    assert success == False
    assert message == "ID sudah ada sebelumnya."

    cursor = db_connection.cursor()
    cursor.execute("DELETE FROM barang WHERE id_barang = 'ID123'")
    db_connection.commit()
    cursor.close()

def test_hapus_barang(db_connection, controller):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, stok, deskripsi_barang, foto) VALUES (%s, %s, %s, %s, %s, %s)", 
                   (1, '123AB', "Test Barang 1", 100, "Description 1", 'ex1.jpg'))
    db_connection.commit()
    lastid = cursor.lastrowid
    controller.hapus_barang(1, lastid)

    cursor.execute("SELECT COUNT(*) FROM barang WHERE id_barang = %s AND id_warehouse = %s", (lastid, 1,))
    count = cursor.fetchone()[0]
    assert count == 0
    cursor.close()


def test_search_barang(db_connection, controller):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, stok, deskripsi_barang, foto) VALUES (%s, %s, %s, %s, %s, %s)", 
                   (1, 'ABC12', "Test Search", 100, "Description 1", 'ex1.jpg'))
    db_connection.commit()

    results = controller.search_barang(1, "Search")
    assert any("Test Search" in item['nama_barang'] for item in results)

    cursor.execute("DELETE FROM barang WHERE id_barang = 'ABC12'")
    db_connection.commit()
    cursor.close()

def test_lihat_barang(db_connection, controller):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, stok, deskripsi_barang, foto) VALUES (%s, %s, %s, %s, %s, %s)", 
                   (1, 'ID127', "View Test Barang", 100, "Description 1", 'ex1.jpg'))
    db_connection.commit()
    barang_id = "ID127"

    result = controller.lihat_barang(1, barang_id)
    assert result is not None
    assert result['nama_barang'] == "View Test Barang"

    cursor.execute("DELETE FROM barang WHERE id_barang = 'ID127'")
    db_connection.commit()
    cursor.close()


def test_pindah_barang(db_connection, controller):
    success, message = controller.pindah_barang("ID131", 1, 2)
    assert success == True
    assert message == "Item is being moved to the new warehouse."

def test_pindahkan_semua_barang(db_connection, controller):
    success, message = controller.pindahkan_semua_barang(1, 2)
    assert success == True
    assert message == "All items are being moved to the new warehouse."

if __name__ == "__main__":
    pytest.main()