# tests/test_list_barang.py

import pytest
import mysql.connector
from classes.ListBarang import ListBarang

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

def test_get_list_barang(db_connection):
    cursor = db_connection.cursor(dictionary=True)
    
    # Insert test data into barang table
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto, status_pengiriman) VALUES (%s, %s, %s, %s, %s, %s, %s)", 
                   (1, "B001", "Test Barang 1", "Deskripsi 1", 10, None, 'Berada di Warehouse'))
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto, status_pengiriman) VALUES (%s, %s, %s, %s, %s, %s, %s)", 
                   (1, "B002", "Test Barang 2", "Deskripsi 2", 20, None, 'Dalam Pengiriman'))
    cursor.execute("INSERT INTO barang (id_warehouse, id_barang, nama_barang, deskripsi_barang, stok, foto, status_pengiriman) VALUES (%s, %s, %s, %s, %s, %s, %s)", 
                   (2, "B003", "Test Barang 3", "Deskripsi 3", 30, None, 'Berada di Warehouse'))
    db_connection.commit()

    # Test retrieving items for a specific warehouse
    results = ListBarang.get_list_barang(db_connection, 1)
    
    cursor.execute("SELECT * FROM barang WHERE id_warehouse = %s", (1,))
    expected_results = cursor.fetchall()
    
    assert len(results) == len(expected_results)
    for result, expected in zip(results, expected_results):
        assert result['id_barang'] == expected['id_barang']
        assert result['id_warehouse'] == expected['id_warehouse']
        assert result['nama_barang'] == expected['nama_barang']
        assert result['deskripsi_barang'] == expected['deskripsi_barang']
        assert result['stok'] == expected['stok']
        assert result['foto'] == expected['foto']
        assert result['status_pengiriman'] == expected['status_pengiriman']

    # Clean up
    cursor.execute("DELETE FROM barang WHERE nama_barang LIKE '%Test%'")
    db_connection.commit()
    cursor.close()

if __name__ == "__main__":
    pytest.main()
