# tests/test_warehouse_controller.py

import pytest
import mysql.connector
from classes.Warehouse import Warehouse
from classes.WarehouseController import WarehouseController

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def controller():
    return WarehouseController(**DB_CONFIG)

def test_tambah_warehouse(db_connection, controller):
    cursor = db_connection.cursor()
    
    cursor.execute("DELETE FROM warehouse WHERE nama_warehouse = %s", ("Test Add Warehouse",))
    db_connection.commit()
    
    success, message = controller.tambah_warehouse("Test Add Warehouse", 100, "Test Address")
    assert success == True
    assert message == "Warehouse berhasil ditambahkan."
    
    success, message = controller.tambah_warehouse("Test Add Warehouse", 100, "Test Address")
    assert success == False
    assert message == "Warehouse dengan nama tersebut sudah ada. Silakan gunakan nama lain."
    
    # Clean up
    cursor.execute("DELETE FROM warehouse WHERE nama_warehouse = %s", ("Test Add Warehouse",))
    db_connection.commit()
    cursor.close()

def test_hapus_warehouse(db_connection, controller):
    cursor = db_connection.cursor()
    
    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Delete Warehouse", 100, "Test Address", 50, 'hoseaganteng@gmail.com'))
    db_connection.commit()
    lastid = cursor.lastrowid
    
    controller.hapus_warehouse(lastid)
    
    cursor.execute("SELECT COUNT(*) FROM warehouse WHERE id_warehouse = %s", (lastid,))
    count = cursor.fetchone()[0]
    assert count == 0

    cursor.close()

def test_lihat_daftar_warehouse(db_connection, controller):
    cursor = db_connection.cursor(dictionary=True)

    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Warehouse 1", 100, "Test Address 1", 50, 'hoseaganteng@gmail.com'))
    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Warehouse 2", 200, "Test Address 2", 100, 'hoseaganteng@gmail.com'))
    db_connection.commit()

    results = controller.lihat_daftar_warehouse()
    
    cursor.execute("SELECT * FROM warehouse WHERE nama_warehouse IN (%s, %s)", ("Test Warehouse 1", "Test Warehouse 2"))
    expected_results = cursor.fetchall()
    
    assert len(results) >= len(expected_results)
    for expected in expected_results:
        assert any(result['id_warehouse'] == expected['id_warehouse'] and
                   result['nama_warehouse'] == expected['nama_warehouse'] and
                   result['kapasitas_maksimum'] == expected['kapasitas_maksimum'] and
                   result['alamat'] == expected['alamat'] and
                   result['kapasitas_terkini'] == expected['kapasitas_terkini']
                   for result in results)

    # Clean up
    cursor.execute("DELETE FROM warehouse WHERE nama_warehouse IN (%s, %s)", ("Test Warehouse 1", "Test Warehouse 2"))
    db_connection.commit()
    cursor.close()


def test_lihat_warehouse(db_connection, controller):
    result = controller.lihat_warehouse(1)
    assert result is not None
    assert 'id_warehouse' in result
    assert result['id_warehouse'] == 1

    non_existent_id = 999999  
    result = controller.lihat_warehouse(non_existent_id)
    assert result is None 

    invalid_id = "invalid_id"
    try:
        result = controller.lihat_warehouse(invalid_id)
        assert result is None
    except Exception as e:
        assert str(e) == "Invalid warehouse ID type"

def test_total_warehouse(db_connection, controller):
    cursor = db_connection.cursor()

    cursor.execute("SELECT COUNT(*) FROM warehouse")
    actual_count = cursor.fetchone()[0]

    result = controller.total_warehouse()
    assert result == actual_count

    cursor.close()

if __name__ == "__main__":
    pytest.main()
